#ifndef MODULES_DPC_UTILS_INCLUDE_SKELETONWRAPPER_H_
#define MODULES_DPC_UTILS_INCLUDE_SKELETONWRAPPER_H_

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/log/logging.h"

#include <string>

#include "PubDataWrapper.h"

template <typename PublisherType, typename EventType, typename DataType>
class PubWrapper
{
private:
	std::string _name;
	PubDataWrapper<DataType>& _dataWrapper;
	ara::log::Logger& _log;
	bool _isInitialized;


	virtual EventType& GetEvent() = 0;

	ara::com::SampleAllocateePtr<DataType> Allocate()
	{
		return GetEvent().Allocate();
	}

protected:
	

public:
	std::shared_ptr<PublisherType> _skeleton;
	PubWrapper<PublisherType, EventType, DataType>(PubDataWrapper<DataType>& dataWrapper, const std::string& name, ara::log::Logger& log) : _dataWrapper(dataWrapper), _name(name), _log(log)
	{
		_isInitialized = false;
		_log.LogInfo() << "Skeleton for: " << _name << " created";
	}

	virtual ~PubWrapper<PublisherType, EventType, DataType>(){}

	void Init()
	{
		_log.LogInfo() << "Initializing skeleton for: " << _name;
		// _skeleton = std::make_shared<PublisherType>(ara::core::InstanceSpecifier(_name.c_str()),
		// 		ara::com::MethodCallProcessingMode::kEvent);
		_skeleton = std::make_shared<PublisherType>(ara::com::InstanceIdentifier("3"),
				ara::com::MethodCallProcessingMode::kEvent);				
		_skeleton->OfferService();
		_isInitialized = true;
		_log.LogInfo() << "Skeleton for: " << _name << " initialized";
	}

	void Init(std::shared_ptr<PublisherType> skeleton)
	{
		_skeleton = skeleton;
		_isInitialized = true;
		_log.LogInfo() << "Skeleton for: " << _name << " initialized";
	}

	void Publish()
	{
		if(!_isInitialized)
		{
			_log.LogError() << "Skeleton must be initialized before it is used @ " << _name ;
			return;
		}
		auto data = Allocate();
		_dataWrapper.GetData(data);
		GetEvent().Send(std::move(data));
	}
};


#endif /* MODULES_DPC_UTILS_INCLUDE_SKELETONWRAPPER_H_ */
